import pandas

import sqlalchemy

from nbtk.jwa.connector.connector import Connector, ConnectorConfiguration


class DatabaseConfiguration(ConnectorConfiguration):

    def __init__(
            self,
            host: str,
            port: int,
            username: str,
            password: str,
            database_name: str,
    ) -> None:
        super().__init__()
        self._host = host
        self._port = port
        self._username = username
        self._password = password
        self._database_name = database_name

    def host(self):
        return self._host

    def port(self):
        return self._port

    def username(self):
        return self._username

    def password(self):
        return self._password

    def database_name(self):
        return self._database_name


class DatabaseConnector(Connector):

    def __init__(
            self,
            conf: DatabaseConfiguration,
            read_sql: str = None,
            write_table_name: str = None,
            if_write_table_exists: str = "fail",
    ) -> None:
        super().__init__()
        self._conf = conf
        self._read_sql = read_sql
        self._write_table_name = write_table_name
        self._if_write_table_exists = if_write_table_exists

    def read_as_data_frame(self) -> pandas.DataFrame:
        with self._connect() as connection:
            return pandas.read_sql(self._read_sql, connection)

    def write_data_frame(self, data: pandas.DataFrame) -> None:
        with self._connect() as connection:
            data.to_sql(self._write_table_name, connection, if_exists=self._if_write_table_exists)

    def _connect(self):
        sql_engine = sqlalchemy.create_engine(self._connection_string())
        return sql_engine.connect()

    def _connection_string(self):
        raise Exception("not implement")



import os
import unittest

from nbtk.jwa.algorithm.k_means import KMeans
from nbtk.jwa.configuration.yaml_configuration import YamlConfiguration
from nbtk.jwa.connector.factory import ConnectorFactory
from nbtk.jwa.statics import root_path


class TestStringMethods(unittest.TestCase):

    def test(self):
        config_path = os.environ.get("CONFIG_PATH", os.path.join(root_path, "config.yaml"))
        configuration_map = YamlConfiguration(config_path).as_map()
        connector_factory = ConnectorFactory(configuration_map["connector"])
        input_connector = connector_factory.input_connector()
        output_connector = connector_factory.output_connector()
        # TODO algorithm should from factory which will generate the right algorithm instance base on type
        KMeans(configuration_map["algorithm"]).run(input_connector, output_connector)


if __name__ == '__main__':
    unittest.main()



"""
The ``nbtk.nb_model`` module provides the definition of a "nebula model".
This module provides functions of setting/loading variables of a model

@author: Zhilin
@date: 21/12/10
@version: v0.1
"""

import logging
import yaml
import json


class Model(yaml.YAMLObject):
	yaml_tag = 'nebula_model'

	def __init__(
		self,
	    name,
		author=None,
	    version=None,
	    type=None,
	    field=None,
	    dependency_command=None,
	    train_req=None,
	    inference_req=None,
	    param_json=[{"formItem": [], "formData": {}, "intro": ""}],
	    metrics={},
	    summary=None,
	    keyword=None,
	    desc=None,
	    reference=None,
	    cite=None,
		output=None
	):
		self.name = name
		self.author = author
		self.version = version
		self.type = type
		self.field = field
		self.dependency_command = dependency_command
		self.train_req = train_req
		self.inference_req = inference_req
		self.param_json = param_json
		self.metrics = metrics
		self.summary = summary
		self.keyword = keyword
		self.desc = desc
		self.reference = reference
		self.cite = cite
		self.output = output

	def log_metric(self, metric, value):
		"""
		Record one metric in the model configuration file (model.yml).

		: param metric: Metric name to be displayed on the model evaluation interface.
		: param value: Exact value of the metric.
		"""
		if metric in self.metrics.keys():
			self.metrics[metric].append(value)
		else:
			self.metrics[metric] = [value]
		return self

	def def_param(
		self,
		name,
		value="",
		type="text",
		max=None,
		min=None,
		precision=None,
		relyBind=None,
		props=None,
		condition=None,
		display_name=None,
		message=None,
		rules=None
	):
		"""
		Define a model parameter, including its name, interaction type, description, default_value
		and value selection bounds. For pro developers, more interactions are provided, such as the rely relationship,
		the condition of hiding/showing the parameter, etc.

		: param name: parameter name(as a varaible name)
		: param value: Exact value of the metric.
		"""
		if display_name is None:
			display_name = name

		if props is None and max is not None and min is not None:
			if precision is None:
				precision = 0
			props = {"max":max, "min":min, "precision":precision}

		item = {"chineseName": display_name, "name": name, "type": type, "props": props, "message": message}

		if rules is not None:
			item["rules"] = rules

		if relyBind is not None:
			item["relyBind"] = relyBind

		#update formItem
		self.param_json[0]["formItem"].append(item)

		#update condition
		if condition is not None:
			if "condition" in self.param_json[0].keys():
				self.param_json[0]["condition"].append(condition)
			else:
				self.param_json[0]["condition"] = [condition]

		#update formData
		self.param_json[0]["formData"][name] = value
		return self

	def set_name(self, name):
		"""
		Set model name

		: param name: name to be set
		"""
		self.name = name
		return self

	def gen_config(self, file="my_model.yml"):
		"""
		Export the nb model object to a yaml configuration file

		: param file: "File_path" to dump the yaml-formatted model
		"""

		if self.desc:
			self.param_json[0]["intro"] = self.desc

		json_file_name = file.split(".yml")[0] + "_params.json"
		with open(json_file_name, 'w') as param_json:
			json.dump(self.param_json, param_json)

		self.param_json = json_file_name

		with open(file, 'w') as config_file:
			yaml.dump(self, config_file, default_flow_style=False)

	def load_config(self, file="my_model.yml"):
		model_loaded = self
		try:
			with open(file, 'r') as config_file:
				model_loaded = yaml.load(config_file)
		except Exception as e:
			print(e)
		finally:
			return model_loaded



# ----------------------------------------Development_and_test------------------------------------------------

_logger = log.getLogger(__name__)

model = Model(name="Model Name")

model.log_metric("r2", 0.923)
model.log_metric("r2", 0.9323)
model.log_metric("r2", 0.9243)
model.log_metric("r2", 0.92123)

model.log_metric("mse", 0.21)
model.def_param(name="columns", type="select", value=2, min=10, max=100)
import argparse
parser = argparse.ArgumentParser()
parser.add_argument("--s", type=str, required=True)
parser.add_argument("--n", type=int, default=2)

args = parser.parse_args()

model.output = {"df":"res.csv"}

model.gen_config()
#
# model.load_config("fsdaf").set_name("second").gen_config("my_model2.yml")




import pandas


class ConnectorConfiguration:

    def __init__(self) -> None:
        super().__init__()


class Connector:

    def __init__(self) -> None:
        super().__init__()

    def read_as_data_frame(self) -> pandas.DataFrame:
        pass

    def write_data_frame(self, df: pandas.DataFrame) -> None:
        pass



from nbtk.jwa.connector.connector import Connector
from nbtk.jwa.connector.database import DatabaseConfiguration
from nbtk.jwa.connector.mysql import MysqlConnector
from nbtk.jwa.connector.postgresql import PostgresqlConnector


class ConnectorFactory:
    def __init__(self, configuration_map: dict) -> None:
        super().__init__()
        self._configuration_map = configuration_map

    def input_connector(self) -> Connector:
        input_connector_map = self._configuration_map["input"]
        if input_connector_map is None:
            raise Exception("input is None")
        return self._extract_connector(input_connector_map)

    def output_connector(self) -> Connector:
        input_connector_map = self._configuration_map["output"]
        if input_connector_map is None:
            raise Exception("input is None")
        return self._extract_connector(input_connector_map)

    def _extract_connector(self, connector_map: dict) -> Connector:
        connector_type = connector_map["type"]
        if "mysql" == connector_type:
            return self._mysql_connector(connector_map)
        if "postgresql" == connector_type:
            return self._postgresql_connector(connector_map)
        raise Exception("type({}) not support".format(connector_type))

    def _mysql_connector(self, connector_map: dict):
        return MysqlConnector(
            conf=self._extract_database_configuration(connector_map),
            read_sql=connector_map["read_sql"],
            write_table_name=connector_map["write_table_name"],
            if_write_table_exists=connector_map.get("if_write_table_exists", "fail"),
        )

    def _postgresql_connector(self, connector_map: dict):
        return PostgresqlConnector(
            conf=self._extract_database_configuration(connector_map),
            read_sql=connector_map.get("read_sql", None),
            write_table_name=connector_map.get("write_table_name", None),
            if_write_table_exists=connector_map.get("if_write_table_exists", "fail"),
        )

    def _extract_database_configuration(self, connector_map: dict):
        return DatabaseConfiguration(
            host=connector_map["host"],
            port=connector_map["port"],
            username=connector_map["username"],
            password=connector_map["password"],
            database_name=connector_map["database_name"],
        )



#!/usr/bin/env python
# -*- coding:utf-8 -*-


import logging
import os
from pathlib import Path

import colorlog
from logging.handlers import RotatingFileHandler
from datetime import datetime

import statics

log_path = os.environ.get("LOG_PATH", os.path.join(statics.root_path, "logs"))
Path(log_path).mkdir(parents=True, exist_ok=True)

log_colors_config = {
    # 终端输出日志颜色配置
    'DEBUG': 'white',
    'INFO': 'cyan',
    'WARNING': 'yellow',
    'ERROR': 'red',
    'CRITICAL': 'bold_red',
}

default_formats = {
    # 终端输出格式
    'color_format': '%(log_color)s%(asctime)s-%(name)s-%(filename)s-[line:%(lineno)d]-%(levelname)s-[日志信息]: %(message)s',
    # 日志输出格式
    'log_format': '%(asctime)s-%(name)s-%(filename)s-[line:%(lineno)d]-%(levelname)s-[日志信息]: %(message)s'
}


class LogHandler:
    """
    先创建日志记录器（logging.getLogger），然后再设置日志级别（logger.setLevel），
    接着再创建日志文件，也就是日志保存的地方（logging.FileHandler），然后再设置日志格式（logging.Formatter），
    最后再将日志处理程序记录到记录器（addHandler）
    """

    def __init__(self):
        self.DEBUG = 'debug'
        self.INFO = 'info'
        self.WARNING = 'warning'
        self.ERROR = 'error'
        self.CRITICAL = 'critical'
        self.__now_time = datetime.now().strftime('%Y-%m-%d')  # 当前日期格式化
        self.__all_log_path = os.path.join(log_path, self.__now_time + "-all" + ".log")  # 收集所有日志信息文件
        self.__error_log_path = os.path.join(log_path, self.__now_time + "-error" + ".log")  # 收集错误日志信息文件
        self.__logger = logging.getLogger()  # 创建日志记录器
        self.__logger.setLevel(logging.DEBUG)  # 设置默认日志记录器记录级别

    @staticmethod
    def __init_logger_handler(log_path):
        """
        创建日志记录器handler，用于收集日志
        :param log_path: 日志文件路径
        :return: 日志记录器
        """
        # 写入文件，如果文件超过1M大小时，切割日志文件，仅保留3个文件
        logger_handler = RotatingFileHandler(filename=log_path, maxBytes=1 * 1024 * 1024, backupCount=3,
                                             encoding='utf-8')
        return logger_handler

    @staticmethod
    def __init_console_handle():
        """创建终端日志记录器handler，用于输出到控制台"""
        console_handle = colorlog.StreamHandler()
        return console_handle

    def __set_log_handler(self, logger_handler, level=logging.DEBUG):
        """
        设置handler级别并添加到logger收集器
        :param logger_handler: 日志记录器
        :param level: 日志记录器级别
        """
        logger_handler.setLevel(level=level)
        self.__logger.addHandler(logger_handler)

    def __set_color_handle(self, console_handle):
        """
        设置handler级别并添加到终端logger收集器
        :param console_handle: 终端日志记录器
        :param level: 日志记录器级别
        """
        console_handle.setLevel(logging.DEBUG)
        self.__logger.addHandler(console_handle)

    @staticmethod
    def __set_color_formatter(console_handle, color_config):
        """
        设置输出格式-控制台
        :param console_handle: 终端日志记录器
        :param color_config: 控制台打印颜色配置信息
        :return:
        """
        formatter = colorlog.ColoredFormatter(default_formats["color_format"], log_colors=color_config)
        console_handle.setFormatter(formatter)

    @staticmethod
    def __set_log_formatter(file_handler):
        """
        设置日志输出格式-日志文件
        :param file_handler: 日志记录器
        """
        formatter = logging.Formatter(default_formats["log_format"], datefmt='%a, %d %b %Y %H:%M:%S')
        file_handler.setFormatter(formatter)

    @staticmethod
    def __close_handler(file_handler):
        """
        关闭handler
        :param file_handler: 日志记录器
        """
        file_handler.close()

    def __console(self, level, message):
        """构造日志收集器"""
        all_logger_handler = self.__init_logger_handler(self.__all_log_path)  # 创建日志文件
        error_logger_handler = self.__init_logger_handler(self.__error_log_path)
        console_handle = self.__init_console_handle()

        self.__set_log_formatter(all_logger_handler)  # 设置日志格式
        self.__set_log_formatter(error_logger_handler)
        self.__set_color_formatter(console_handle, log_colors_config)

        self.__set_log_handler(all_logger_handler)  # 设置handler级别并添加到logger收集器
        self.__set_log_handler(error_logger_handler, level=logging.ERROR)
        self.__set_color_handle(console_handle)

        if level == 'info':
            self.__logger.info(message)
        elif level == 'debug':
            self.__logger.debug(message)
        elif level == 'warning':
            self.__logger.warning(message)
        elif level == 'error':
            self.__logger.error(message)
        elif level == 'critical':
            self.__logger.critical(message)

        self.__logger.removeHandler(all_logger_handler)  # 避免日志输出重复问题
        self.__logger.removeHandler(error_logger_handler)
        self.__logger.removeHandler(console_handle)

        self.__close_handler(all_logger_handler)  # 关闭handler
        self.__close_handler(error_logger_handler)

    def debug(self, message):
        self.__console('debug', message)

    def info(self, message):
        self.__console('info', message)

    def warning(self, message):
        self.__console('warning', message)

    def error(self, message):
        self.__console('error', message)

    def critical(self, message):
        self.__console('critical', message)

    # mysql日志单独提出来
    def mysql(self, level, message):
        self.__console(level, message)

    # greenplum日志单独提出来
    def gp(self, level, message):
        self.__console(level, message)

    def get_log(self):
        return self


if __name__ == '__main__':
    for i in range(50):
        LogHandler().get_log().info("这是日志信息")
        LogHandler().get_log().debug("这是debug信息")
        LogHandler().get_log().warning("这是警告信息")
        LogHandler().get_log().error("这是错误日志信息")
        LogHandler().get_log().critical("这是严重级别信息")
        LogHandler().get_log().gp("info", "asdasda")



from urllib import parse

from nbtk.jwa.connector.database import DatabaseConfiguration, DatabaseConnector


class PostgresqlConnector(DatabaseConnector):

    def __init__(
            self,
            conf: DatabaseConfiguration,
            read_sql: str = None,
            write_table_name: str = None,
            if_write_table_exists: str = "fail"
    ) -> None:
        super().__init__(conf, read_sql, write_table_name, if_write_table_exists)

    def _connection_string(self):
        return "postgresql+psycopg2://{}:{}@{}:{}/{}".format(
            self._conf.username(),
            parse.quote_plus(self._conf.password()),
            self._conf.host(),
            self._conf.port(),
            self._conf.database_name(),
        )



from sklearn.cluster import MiniBatchKMeans

from nbtk.jwa.connector.connector import Connector
from nbtk.jwa.log.log_handler import LogHandler


class KMeans:
    def __init__(self, configuration: dict) -> None:
        super().__init__()
        self._logger = LogHandler()
        self._cols = configuration["cols"]
        self._n_clusters: int = int(configuration["n_clusters"])
        self._max_iter: int = int(configuration["max_iter"])
        self._batch_size: int = int(configuration.get("batch_size", 10000))
        self._cluster_id_column_name = configuration.get("cluster_id_column_name", "_cluster_id_")

    def run(self, connector_in: Connector, connector_out: Connector):
        data = connector_in.read_as_data_frame()
        data = data.dropna()
        ml_model = MiniBatchKMeans(
            n_clusters=self._n_clusters,
            max_iter=self._max_iter,
            batch_size=self._batch_size
        ).fit(data[self._cols].values)
        pred = ml_model.labels_
        data[self._cluster_id_column_name] = pred.astype("int")
        self._logger.info(data)
        connector_out.write_data_frame(data)



import yaml


class YamlConfiguration:
    def __init__(self, path: str) -> None:
        with open(path, "r") as stream:
            self._configuration_map = yaml.safe_load(stream)

    def as_map(self):
        return self._configuration_map



import time
import re
from dateutil.parser import parse
from nbtk.jwa.log import log_handler

TIMESTAMP_10 = r"^\d{10}(\.\d+)?$"
TIMESTAMP_13 = r"^\d{13}$"

log = log_handler.LogHandler().get_log()

def get_err_msg():
    msg = py_to_java(str({"error_msg": "error in saving and updating", "status": "FAIL"}))
    log.error(msg)
    return msg


def err_formatter(e):
    msg = py_to_java(str({"error_msg": str(e).replace("'", "_").replace('"', "_")}))
    log.error(msg)
    return msg


def format_time(data, pattern):
    if data == None:
        return None

    data = data.replace("年", "-").replace("月", "-").replace(",", ", ")
    data = re.compile(
        r"日|Mon\.|Tue\.|Wed\.|Thur\.|Fri\.|Sat\.|Sun\.|Monday|Tuesday|Wednesday|Thursday|Friday|Saturday|Sunday").sub(
        "", data)
    if pattern == "timestamp":
        if re.match(TIMESTAMP_10, data):
            ret = data
        elif re.match(TIMESTAMP_13, data):
            ret = int(data) / 1000
        else:
            pattern = "%Y-%m-%d %H:%M:%S"
            ret = time.mktime(time.strptime(parse(data).strftime(pattern), pattern))
        return int(ret)
    else:
        if re.match(TIMESTAMP_10, data):
            return time.strftime(pattern, time.localtime(int(data)))
        elif re.match(TIMESTAMP_13, data):
            return time.strftime(pattern, time.localtime(float(data) / 1000))
        else:
            return parse(data).strftime(pattern)


def py_to_java(s):
    return s.replace("'", '"') \
        .replace("False", "false") \
        .replace("True", "true") \
        .replace(", ", ",") \
        .replace(": ", ":") \
        .replace("None", "null") \
        .replace("nan", "null")
        # .replace("inf", "2147483647")


def java_to_py(java_string):
    py_string = java_string.replace("'", '"')
    return py_string


def cast_float(obj):
    if isinstance(obj, float):
        return round(obj, 3)
    elif isinstance(obj, dict):
        return dict((k, cast_float(v)) for k, v in obj.items())
    elif isinstance(obj, list):
        return list(map(cast_float, obj))
    elif isinstance(obj, tuple):
        return tuple(map(cast_float, obj))
    else:
        return obj


def dup_name_handler(col_name, df_col):
    i = 1
    original_col_name = col_name
    while col_name in df_col:
        col_name = original_col_name + "_" + str(i)
        i += 1
    return col_name
